<?xml version="1.0" encoding="UTF-8"?>
<!-- Validation file based on Schematron for MPFO-XML Strict Content MathML

Copyright: German publicly funded research project autoSWIFT (2015-2018), BMBF 16ES03, https://www.edacentrum.de/autoswift/en/content/over-project

THIS SOFTWARE / FILE FORMAT IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

-->
<sch:schema xmlns:sch="http://purl.oclc.org/dsdl/schematron">
    <sch:ns uri="http://www.mpfo.org/mpfo-0.7" prefix="mpfo"/>
    <sch:ns uri="http://www.w3.org/1998/Math/MathML" prefix="mml"/>
    <!-- Get namespace URI of the document's root element. -->
    <sch:let name="mpfo_uri" value="namespace-uri(/*)"/>
    <sch:pattern>
        <sch:title>Existence of each root element</sch:title>
        <sch:rule context="mpfo:MPFo">
            <sch:assert test="mpfo:Description or mpfo:Include/mpfo:ExternalDefinition/mpfo:Description">No Description element could be found in top level or Include element.</sch:assert>
            <sch:assert test="mpfo:PortDefinition or mpfo:Include/mpfo:ExternalDefinition/mpfo:PortDefinition">No PortDefinition element could be found in top level or Include element.</sch:assert>
            <sch:assert test="mpfo:LoadDefinition or mpfo:Include/mpfo:ExternalDefinition/mpfo:LoadDefinition">No LoadDefinition element could be found in top level or Include element.</sch:assert>
            <sch:assert test="mpfo:ActionDefinition or mpfo:Include/mpfo:ExternalDefinition/mpfo:ActionDefinition">No ActionDefinition element could be found in top level or Include element.</sch:assert>
            <sch:assert test="mpfo:ActivityDefinition or mpfo:Include/mpfo:ExternalDefinition/mpfo:ActivityDefinition">No ActivityDefinition element could be found in top level or Include element.</sch:assert>
            <sch:assert test="mpfo:ScenarioDefinition or mpfo:Include/mpfo:ExternalDefinition/mpfo:ScenarioDefinition">No ScenarioDefinition element could be found in top level or Include element.</sch:assert>
        </sch:rule>
    </sch:pattern>
    <sch:pattern>
        <sch:title>Existence of referenced elements</sch:title>
        <sch:rule context="mpfo:Document">
            <sch:assert test="not(@PredecessorDocumentRef) or @PredecessorDocumentRef = //mpfo:Document/@Id">No Document element with Id "<sch:value-of select="@PredecessorDocumentRef"/>" could be found.</sch:assert>
        </sch:rule>
        <sch:rule context="mpfo:PredecessorActivityRef">
            <sch:assert test="@Ref = //mpfo:Activity/@Id">No Activity element with Id "<sch:value-of select="@Ref"/> could be found.</sch:assert>
        </sch:rule>
        <!-- Define rule that checks if there is a <mpfo:NAME> element with matching @Id for each <mpfo:NAMERef> element. --> 
        <sch:rule context="mpfo:ActionRef|mpfo:ActionSetRef|
                           mpfo:ActivityFlowRef|mpfo:ActivityFlowSetRef|
                           mpfo:ActivityRef|
                           mpfo:LoadRef|mpfo:LoadSetRef|
                           mpfo:PortRef|mpfo:PortSetRef|
                           mpfo:StringRef|mpfo:StringVectorRef|mpfo:StringSparseVectorRef|mpfo:StringMatrixRef|mpfo:StringSparseMatrixRef|
                           mpfo:SNumberRef|mpfo:MNumberRef|
                           mpfo:NumberVectorRef|mpfo:NumberSparseVectorRef|mpfo:NumberMatrixRef|mpfo:NumberSparseMatrixRef|
                           mpfo:NumberIntervalRef|mpfo:NumberIntervalVectorRef|
                           mpfo:ToleranceRef|mpfo:ToleranceVectorRef|mpfo:ToleranceSparseVectorRef|mpfo:ToleranceMatrixRef|mpfo:ToleranceSparseMatrixRef|
                           mpfo:TolerancedNumberRef|mpfo:TolerancedNumberIntervalRef|
                           mpfo:TolerancedNumberVectorRef|mpfo:TolerancedNumberSparseVectorRef|mpfo:TolerancedNumberMatrixRef|mpfo:TolerancedNumberSparseMatrixRef|
                           mpfo:UnitRef|mpfo:UnitVectorRef|mpfo:UnitSparseVectorRef|mpfo:UnitMatrixRef|mpfo:UnitSparseMatrixRef|
                           mpfo:QuantityRef|mpfo:QuantityIntervalRef|
                           mpfo:QuantityVectorRef|mpfo:QuantitySparseVectorRef|mpfo:QuantityMatrixRef|mpfo:QuantitySparseMatrixRef|
                           mpfo:TolerancedQuantityRef|mpfo:TolerancedQuantityIntervalRef|
                           mpfo:TolerancedQuantityVectorRef|mpfo:TolerancedQuantitySparseVectorRef|mpfo:TolerancedQuantityMatrixRef|mpfo:TolerancedQuantitySparseMatrixRef|
                           mpfo:ArgumentRef|mpfo:ArgumentWithUnitRef|
                           mpfo:ParameterRef|mpfo:ParameterWithUnitRef|
                           mpfo:ParameterValueRef|mpfo:ParameterValueMagnitudeRef|
                           mpfo:FormulaTemplateRef|mpfo:FormulaRef|
                           mpfo:SymbolicFunctionTemplateRef|mpfo:SymbolicFunctionRef|
                           mpfo:TableFunctionRef">
            <!-- Get the element name we are looking for by removing the string 'Ref' from the end of the current element name. -->
            <sch:let name="element_name" value="substring-before(local-name(), 'Ref')"/>
            <!-- Check if there is an element with that name and matching @Id. -->
            <sch:assert test="@Ref = //*[namespace-uri() = $mpfo_uri and local-name() = $element_name]/@Id">No <sch:value-of select="$element_name"/> element with Id "<sch:value-of select="@Ref"/>" could be found.</sch:assert>
        </sch:rule>
    </sch:pattern>
    <sch:pattern>
        <sch:title>MNumber contains cn or apply</sch:title>
        <sch:rule context="mpfo:MNumber/mml:math">
            <sch:assert test="mml:cn or mml:apply"> MNumber must contain mml:cn or mml:apply
                element. </sch:assert>
        </sch:rule>
    </sch:pattern>
    <sch:pattern>
        <sch:title>MNumber rational or complex</sch:title>
        <sch:rule context="mpfo:MNumber/mml:math/mml:apply">
            <sch:assert
                test="count(*) = 3 and (mml:csymbol[@cd = 'nums1' and text() = 'rational']/following-sibling::mml:cn[@type = 'integer']/following-sibling::mml:cn[@type = 'integer' and text() != 0] or mml:csymbol[@cd = 'complex1' and text() = 'complex_cartesian']/following-sibling::mml:cn/following-sibling::mml:cn)"
                > MNumber/mml:apply must define nums1/rational with two integers (2nd != 0) or
                complex1/complex_cartesian with two numbers. </sch:assert>
        </sch:rule>
    </sch:pattern>
    <sch:pattern>
        <sch:title>Number of vector elements</sch:title>
        <sch:rule context="mpfo:TolerancedNumberVector/mpfo:ToleranceVector|
                           mpfo:QuantityVector/mpfo:UnitVector|
                           mpfo:TolerancedQuantityVector/mpfo:ToleranceVector|
                           mpfo:TolerancedQuantityVector/mpfo:UnitVector">
            <sch:assert test="count(parent::*/mpfo:NumberVector/*) = count(*)"> Corresponding
                vectors must have the same number of elements. </sch:assert>
        </sch:rule>
    </sch:pattern>
    <sch:pattern>
        <sch:title>Index of sparse vector elements</sch:title>
        <sch:rule context="mpfo:StringSparseVector/mpfo:Elements/mpfo:Element|
                           mpfo:NumberSparseVector/mpfo:Elements/mpfo:Element|
                           mpfo:ToleranceSparseVector/mpfo:Elements/mpfo:Element|
                           mpfo:TolerancedNumberSparseVector/mpfo:Elements/mpfo:Element|
                           mpfo:UnitSparseVector/mpfo:Elements/mpfo:Element|
                           mpfo:QuantitySparseVector/mpfo:Elements/mpfo:Element|
                           mpfo:TolerancedQuantitySparseVector/mpfo:Elements/mpfo:Element|
                           mpfo:ColumnUnits/mpfo:Elements/mpfo:Element">
            <sch:assert test="@Index &lt;= parent::*/parent::*/@ElementCount"> Index of sparse vector elements
                must not be greater than the element count of the matrix.</sch:assert>
        </sch:rule>
    </sch:pattern>
    <sch:pattern>
        <sch:title>Unique indices of sparse vector elements</sch:title>
        <sch:rule context="mpfo:StringSparseVector/mpfo:Elements|
                           mpfo:NumberSparseVector/mpfo:Elements|
                           mpfo:ToleranceSparseVector/mpfo:Elements|
                           mpfo:TolerancedNumberSparseVector/mpfo:Elements|
                           mpfo:UnitSparseVector/mpfo:Elements|
                           mpfo:QuantitySparseVector/mpfo:Elements|
                           mpfo:TolerancedQuantitySparseVector/mpfo:Elements|
                           mpfo:ColumnUnits/mpfo:Elements">
            <sch:assert test="count(*) = count(*[not(@Index=preceding-sibling::*/@Index)])">
                Not all indices of the sparse vector elements are unique.</sch:assert>
        </sch:rule>
    </sch:pattern>
    <sch:pattern>
        <sch:title>Equal matrix row size</sch:title>
        <sch:rule context="mpfo:StringMatrixRow|
                           mpfo:NumberMatrixRow|
                           mpfo:ToleranceMatrixRow|
                           mpfo:UnitMatrixRow">
            <sch:assert test="count(parent::*/*[1]/*) = count(*)"> All rows in a matrix must have
                the same size. </sch:assert>
        </sch:rule>
    </sch:pattern>
    <sch:pattern>
        <sch:title>Matrix size</sch:title>
        <sch:rule context="mpfo:TolerancedNumberMatrix/mpfo:ToleranceMatrix|
                           mpfo:QuantityMatrix/mpfo:UnitMatrix|
                           mpfo:TolerancedQuantityMatrix/mpfo:ToleranceMatrix|
                           mpfo:TolerancedQuantityMatrix/mpfo:UnitMatrix">
            <sch:assert test="count(parent::*/mpfo:NumberMatrix/mpfo:NumberMatrixRow) = count(*)">
                Corresponding matrices must have the same number of rows. </sch:assert>
        </sch:rule>
    </sch:pattern>
    <sch:pattern>
        <sch:title>Matrix row size</sch:title>
        <sch:rule context="mpfo:TolerancedNumberMatrix/mpfo:ToleranceMatrix">
            <sch:assert
                test="count(parent::*/mpfo:NumberMatrix/mpfo:NumberMatrixRow[1]/*) = count(*[1]/*)">
                Rows of corresponding matrices must have the same size. </sch:assert>
        </sch:rule>
    </sch:pattern>
    <sch:pattern>
        <sch:title>Indices of sparse matrix elements</sch:title>
        <sch:rule context="mpfo:StringSparseMatrix/mpfo:Elements/mpfo:Element|
            mpfo:NumberSparseMatrix/mpfo:Elements/mpfo:Element|
            mpfo:ToleranceSparseMatrix/mpfo:Elements/mpfo:Element|
            mpfo:TolerancedNumberSparseMatrix/mpfo:Elements/mpfo:Element|
            mpfo:UnitSparseMatrix/mpfo:Elements/mpfo:Element|
            mpfo:QuantitySparseMatrix/mpfo:Elements/mpfo:Element|
            mpfo:TolerancedQuantitySparseMatrix/mpfo:Elements/mpfo:Element">
            <sch:assert test="@RowIndex &lt;= parent::*/parent::*/@RowCount"> Row index of sparse matrix elements
                must not be greater than the row count of the matrix.</sch:assert>
            <sch:assert test="@ColumnIndex &lt;= parent::*/parent::*/@ColumnCount"> Column index of sparse matrix elements
                must not be greater than the column count of the matrix.</sch:assert>
        </sch:rule>
    </sch:pattern>
    <sch:pattern>
        <sch:title>Unique indices of sparse vector elements</sch:title>
        <sch:rule context="mpfo:StringSparseMatrix/mpfo:Elements|
            mpfo:NumberSparseMatrix/mpfo:Elements|
            mpfo:ToleranceSparseMatrix/mpfo:Elementst|
            mpfo:TolerancedNumberSparseMatrix/mpfo:Elements|
            mpfo:UnitSparseMatrix/mpfo:Elements|
            mpfo:QuantitySparseMatrix/mpfo:Elements|
            mpfo:TolerancedQuantitySparseMatrix/mpfo:Elements">
            <sch:assert test="count(*) = count(*[not(@RowIndex=preceding-sibling::*/@RowIndex and @ColumnIndex=preceding-sibling::*/@ColumnIndex)])">
                Not all combinations of row and column indices of the sparse matrix elements are unique.</sch:assert>
        </sch:rule>
    </sch:pattern>
</sch:schema>
